@@ -12,23 +12,43 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import re
+"""
+Responsible for converting the Sphinx configuration stored by projects and used
+by giza, into a ``sphinx-build`` invocation. See
+:func:`giza.content.sphinx.run_sphinx` for the core of this operation.
+
+:mod:`giza.content.sphinx` also contains:
+
+- the error message processing, which normalizes references to paths,
+  deduplicates log messages given multiple builds or multi-process Sphinx
+  operation, and removes non-actionable messages. See
+  :func:`giza.content.sphinx.output_sphinx_stream()`,
+  :func:`giza.content.sphinx.is_msg_worth()` and
+  :func:`giza.content.sphinx.path_normalization()`.
+
+- the orchestration of all post-processing. See
+  :func:`giza.content.sphinx.finalize_sphinx_build()`.
+"""
+
+import collections
 import logging
-import pkg_resources
 import os.path
-from multiprocessing import cpu_count
+import pkg_resources
+import re
+import sys
 
 logger = logging.getLogger('giza.content.sphinx')
 
 from giza.core.app import BuildApp
 from giza.tools.command import command
-from giza.tools.strings import timestamp
+from giza.tools.files import safe_create_directory
+from giza.tools.timing import Timer
 from giza.config.helper import get_config_paths
 from giza.content.links import create_manual_symlink, get_public_links
-from giza.content.manpages import manpage_url_tasks
 from giza.content.post.json_output import json_output_tasks
 from giza.content.post.singlehtml import finalize_single_html_tasks
 from giza.content.post.archives import man_tarball, html_tarball, get_tarball_name
+from giza.content.post.manpages import manpage_url_tasks
 from giza.content.post.gettext import gettext_tasks
 from giza.content.post.slides import slide_tasks
 from giza.content.post.latex import pdf_tasks
@@ -41,7 +61,10 @@ def is_parallel_sphinx(version):
     return version >= '1.2'
 
 def get_tags(target, sconf):
-    ret = set()
+    if 'tags' in sconf:
+        ret = set(sconf.tags)
+    else:
+        ret = set()
 
     ret.add(target)
     ret.add(target.split('-')[0])
@@ -68,22 +91,29 @@ def get_sphinx_args(sconf, conf):
 
     if is_parallel_sphinx(pkg_resources.get_distribution("sphinx").version):
         if 'serial_sphinx' in conf.runstate:
+            logger.info('running with serial sphinx processes')
             if conf.runstate.serial_sphinx == "publish":
                 if ((len(conf.runstate.builder) >= 1 or 'publish' in conf.runstate.builder) or
                     len(conf.runstate.languages_to_build) >= 1 or
                     len(conf.runstate.editions_to_build) >= 1):
                     pass
                 else:
-                    o.append(' '.join( [ '-j', str(cpu_count()) ]))
+                    o.append(' '.join( [ '-j', str(conf.runstate.pool_size) ]))
             elif conf.runstate.serial_sphinx is False:
-                o.append(' '.join( [ '-j', str(cpu_count()) ]))
+                logger.info('running with parallelized sphinx processes')
+                o.append(' '.join( [ '-j', str(conf.runstate.pool_size) ]))
             elif (isinstance(conf.runstate.serial_sphinx, (int, long, float)) and
                   conf.runstate.serial_sphinx > 1):
+                logger.info('running with parallelized sphinx processes')
                 o.append(' '.join(['-j', str(conf.runstate.serial_sphinx)]))
             else:
                 pass
+        elif len(conf.runstate.builder) >= conf.runstate.pool_size:
+            logger.info('running with serail sphinx processes')
+            pass
         else:
-            o.append(' '.join( [ '-j', str(cpu_count()) ]))
+            logger.info('running with parallelized sphinx processes')
+            o.append(' '.join( [ '-j', str(conf.runstate.pool_size) ]))
 
     o.append(' '.join( [ '-c', conf.paths.projectroot ] ))
 
@@ -132,12 +162,24 @@ def output_sphinx_stream(out, conf):
 
         printable.append(l)
 
-    printable = list(set(printable))
-    printable.sort()
+    printable = stable_deduplicate(printable)
 
     logger.info('sphinx builder has {0} lines of output, processed from {1}'.format(len(printable), len(out)))
     print_build_messages(printable)
 
+def stable_deduplicate(lines):
+    ## this should probably just use OrderedSet() in the future
+
+    mapping = collections.OrderedDict()
+
+    for idx, ln in enumerate(lines):
+        mapping[ln] = idx
+
+    if sys.version_info >= (3, 0):
+        return [ ln for _, ln in mapping.keys() ]
+    else:
+        return mapping.keys()
+
 def print_build_messages(messages):
     for l in ( l for l in messages if l is not None ):
         print(l)
@@ -168,9 +210,13 @@ def is_msg_worthy(l):
         return False
     elif l.endswith('Duplicate ID: "cmdoption-h".'):
         return False
+    elif l.endswith('should look like "opt", "-opt args", "--opt args" or "/opt args" or "+opt args"'):
+        return False
+    elif l.endswith('should look like "-opt args", "--opt args" or "/opt args" or "+opt args"'):
+        return False
     elif l.endswith('should look like "opt", "-opt args", "--opt args" or "/opt args"'):
         return False
-    elif l.endswith('should look like "-opt args", "--opt args" or "/opt args"'):
+    elif 'nonlocal image URI found' in l:
         return False
     else:
         return True
@@ -181,18 +227,16 @@ def printer(string):
 #################### Builder Operation ####################
 
 def run_sphinx(builder, sconf, conf):
-    dirpath = os.path.join(conf.paths.projectroot, conf.paths.branch_output, builder)
-    if not os.path.exists(dirpath):
-        os.makedirs(dirpath)
-        logger.info('created directories "{1}" for sphinx builder {0}'.format(builder, dirpath))
+    if safe_create_directory(sconf.build_output):
+        logger.info('created directory "{1}" for sphinx builder {0}'.format(builder, sconf.build_output))
 
     if 'language' in sconf and sconf.language is not None:
         command('sphinx-intl build --language=' + sconf.language)
         logger.info('compiled all PO files for translated build.')
 
-    logger.info('starting sphinx build {0} at {1}'.format(builder, timestamp()))
+    logger.info('starting sphinx build {0}'.format(builder))
 
-    cmd = 'sphinx-build {0} -d {1}/doctrees-{2} {3} {4}' # per-builder-doctreea
+    cmd = 'sphinx-build {0} -d {1}/doctrees-{2} {3} {4}' # per-builder-doctree
 
     sphinx_cmd = cmd.format(get_sphinx_args(sconf, conf),
                             os.path.join(conf.paths.projectroot, conf.paths.branch_output),
@@ -201,17 +245,19 @@ def run_sphinx(builder, sconf, conf):
                             sconf.build_output)
 
     logger.debug(sphinx_cmd)
-    out = command(sphinx_cmd, capture=True, ignore=True)
-    # out = sphinx_native_worker(sphinx_cmd)
-    logger.info('completed sphinx build {0} at {1}'.format(builder, timestamp()))
+    with Timer("running sphinx build for: {0}, {1}, {2}".format(builder, sconf.language, sconf.edition)):
+        out = command(sphinx_cmd, capture=True, ignore=True)
+
+    logger.info('completed sphinx build {0}'.format(builder))
 
     if True: # out.return_code == 0:
-        logger.info('successfully completed {0} sphinx build at {1}'.format(builder, timestamp()))
+        logger.info('successfully completed {0} sphinx build ({1})'.format(builder, out.return_code))
 
         finalizer_app = BuildApp(conf)
         finalizer_app.root_app = False
         finalize_sphinx_build(sconf, conf, finalizer_app)
-        finalizer_app.run()
+        with Timer("finalize sphinx {0} build".format(builder)):
+            finalizer_app.run()
     else:
         logger.warning('the sphinx build {0} was not successful. not running finalize operation'.format(builder))
 
@@ -239,13 +285,14 @@ def finalize_sphinx_build(sconf, conf, app):
     target = sconf.builder
     logger.info('starting to finalize the Sphinx build {0}'.format(target))
 
-    if target == 'linkcheck':
+    if target == 'html':
         app.pool = 'serial'
         task = app.add('task')
-        task.job = printer
-        task.target = os.path.join(conf.paths.projectroot,
-                                   conf.paths.branch_output, builder, 'output.txt')
-        task.args = '{0}: See {1}/{0}/output.txt for output.'.format(builder, conf.paths.branch_output)
+        task.job = html_tarball
+        task.target = [get_tarball_name('html', conf),
+                       get_tarball_name('link-html', conf)]
+        task.args = [sconf.name, conf]
+        task.description = "creating tarball for html archive"
     elif target == 'dirhtml':
         app.pool = 'thread'
         for job in (finalize_dirhtml_build, error_pages):
@@ -268,7 +315,7 @@ def finalize_sphinx_build(sconf, conf, app):
         app.pool = 'serial'
         task = app.add('task')
         task.job = finalize_epub_build
-        task.args = [target, conf]
+        task.args = (target, conf)
         task.description = 'finalizing epub build'
     elif target == 'man':
         app.pool = 'thread'
@@ -278,15 +325,7 @@ def finalize_sphinx_build(sconf, conf, app):
         task.target = [get_tarball_name('man', conf),
                        get_tarball_name('link-man', conf)]
         task.args = [target, conf]
-        task.description = "creating tarball for manpages"
-    elif target == 'html':
-        app.pool = 'serial'
-        task = app.add('task')
-        task.job = html_tarball
-        task.target = [get_tarball_name('html', conf),
-                       get_tarball_name('link-html', conf)]
-        task.args = [sconf.name, conf]
-        task.description = "creating tarball for html archive"
+        task.description = "creating tarball for 1manpages"
     elif target == 'slides':
         app.pool = 'thread'
         slide_tasks(sconf, conf, app)
@@ -302,3 +341,10 @@ def finalize_sphinx_build(sconf, conf, app):
     elif target == 'gettext':
         app.pool = 'thread'
         gettext_tasks(conf, app)
+    elif target == 'linkcheck':
+        app.pool = 'serial'
+        task = app.add('task')
+        task.job = printer
+        task.target = os.path.join(conf.paths.projectroot,
+                                   conf.paths.branch_output, builder, 'output.txt')
+        task.args = '{0}: See {1}/{0}/output.txt for output.'.format(builder, conf.paths.branch_output)
