@@ -1,3 +1,23 @@
+# Copyright 2014 MongoDB, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+# http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
+"""
+Entry points that run the content generation tasks. Typically use these entry
+points for testing and for prepopulating build directories. These functions do
+not run in common practice.
+"""
+
 import logging
 
 logger = logging.getLogger('giza.operations.generate')
@@ -11,7 +31,6 @@ from giza.tools.files import rm_rf
 from giza.content.assets import assets_tasks, assets_clean
 from giza.content.images import image_tasks, image_clean
 from giza.content.intersphinx import intersphinx_tasks, intersphinx_clean
-from giza.content.options import option_tasks, option_clean
 from giza.content.param import api_tasks, api_clean
 from giza.content.table import table_tasks, table_clean
 from giza.content.toc import toc_tasks, toc_clean
@@ -19,12 +38,20 @@ from giza.content.robots import robots_txt_tasks
 from giza.content.redirects import make_redirect, redirect_tasks
 from giza.content.examples.tasks import example_tasks
 from giza.content.steps.tasks import step_tasks, step_clean
+from giza.content.options.tasks import option_tasks, option_clean
+from giza.content.release.tasks import release_tasks, release_clean
 
 from giza.content.primer import primer_migration_tasks
 from giza.content.primer import clean as primer_clean
 
+from giza.operations.sphinx_cmds import build_content_generation_tasks
+from giza.content.source import source_tasks
+from giza.config.sphinx_config import render_sconf
+from giza.content.dependencies import refresh_dependency_tasks
+
 @argh.arg('--edition', '-e')
 @argh.arg('--clean', '-c', default=False, action="store_true", dest="clean_generated")
+@argh.expects_obj
 def toc(args):
     c = fetch_config(args)
 
@@ -37,6 +64,7 @@ def toc(args):
 
 @argh.arg('--edition', '-e')
 @argh.arg('--clean', '-c', default=False, action="store_true", dest="clean_generated")
+@argh.expects_obj
 def steps(args):
     c = fetch_config(args)
     app = BuildApp(c)
@@ -49,6 +77,7 @@ def steps(args):
     app.run()
 
 @argh.arg('--clean', '-c', default=False, action="store_true", dest="clean_generated")
+@argh.expects_obj
 def options(args):
     c = fetch_config(args)
 
@@ -60,6 +89,7 @@ def options(args):
         app.run()
 
 @argh.arg('--clean', '-c', default=False, action="store_true", dest="clean_generated")
+@argh.expects_obj
 def api(args):
     c = fetch_config(args)
     app = BuildApp(c)
@@ -72,6 +102,7 @@ def api(args):
     app.run()
 
 @argh.arg('--clean', '-c', default=False, action="store_true", dest="clean_generated")
+@argh.expects_obj
 def assets(args):
     c = fetch_config(args)
     app = BuildApp(c)
@@ -84,6 +115,7 @@ def assets(args):
     app.run()
 
 @argh.arg('--clean', '-c', default=False, action="store_true", dest="clean_generated")
+@argh.expects_obj
 def images(args):
     c = fetch_config(args)
     app = BuildApp(c)
@@ -96,6 +128,7 @@ def images(args):
     app.run()
 
 @argh.arg('--clean', '-c', default=False, action="store_true", dest="clean_generated")
+@argh.expects_obj
 def intersphinx(args):
     c = fetch_config(args)
     app = BuildApp(c)
@@ -108,6 +141,7 @@ def intersphinx(args):
     app.run()
 
 @argh.arg('--clean', '-c', default=False, action="store_true", dest="clean_generated")
+@argh.expects_obj
 def primer(args):
     c = fetch_config(args)
     app = BuildApp(c)
@@ -120,6 +154,20 @@ def primer(args):
     app.run()
 
 @argh.arg('--clean', '-c', default=False, action="store_true", dest="clean_generated")
+@argh.expects_obj
+def release(args):
+    c = fetch_config(args)
+    app = BuildApp(c)
+
+    if c.runstate.clean_generated is True:
+        release_clean(c, app)
+    else:
+        release_tasks(c, app)
+
+    app.run()
+
+@argh.arg('--clean', '-c', default=False, action="store_true", dest="clean_generated")
+@argh.expects_obj
 def tables(args):
     c = fetch_config(args)
     app = BuildApp(c)
@@ -131,6 +179,7 @@ def tables(args):
 
     app.run()
 
+@argh.expects_obj
 def examples(args):
     c = fetch_config(args)
     app = BuildApp(c)
@@ -140,6 +189,7 @@ def examples(args):
     app.run()
 
 @argh.arg('--edition', '-e')
+@argh.expects_obj
 def robots(args):
     c = fetch_config(args)
     app = BuildApp(c)
@@ -151,6 +201,7 @@ def robots(args):
 
 @argh.arg('--edition', '-e')
 @argh.arg('--print', '-p', default=False, action='store_true', dest='dry_run')
+@argh.expects_obj
 def redirects(args):
     c = fetch_config(args)
 
@@ -161,3 +212,21 @@ def redirects(args):
 
         redirect_tasks(c, app)
         app.run()
+
+@argh.arg('--edition', '-e')
+@argh.arg('--language', '-l')
+@argh.expects_obj
+def source(args):
+    conf = fetch_config(args)
+    app = BuildApp(conf)
+
+    sconf = render_sconf(args.edition, 'html', args.language, conf)
+
+    prep_app = app.add('app')
+    source_tasks(conf, sconf, prep_app)
+    prep_app.run()
+
+    build_content_generation_tasks(conf, prep_app.add('app'))
+    refresh_dependency_tasks(conf, prep_app.add('app'))
+
+    app.run()
